@@ -851,13 +851,17 @@ class BaseExperiment:
 
     def delete_saved_model(
         self,
-        model_type : "pl.LightningModule",
+        model_type : Union["pl.LightningModule", "nn.Module", "BaseEstimator"],
+        version : int = -1, 
         **hparams
     ):
-        if BaseNet in inspect.getmro(model_type) or ConvNet in inspect.getmro(model_type):
+        if not isinstance(version, int) or version <-1:
+            raise ValueError(f"Wrong value for version. Expected to be an integer greater than or equal to -1. Got {version} instead.")
+        if BaseNet in inspect.getmro(model_type) or ConvNet in inspect.getmro(model_type) or SegNet in inspect.getmro(model_type):
             pseudomodel = model_type(dims=self.dims, n_classes=self.n_classes, **hparams)
         else:
             pseudomodel = model_type(**hparams)
+        self.update_log_path()
         model_metadata = self.set_model_metadata(pseudomodel, hparams)
         del_ckpt_path = f'{self.log_path}/checkpoints/{model_type.__name__}' 
         del_json_path = f'{self.log_path}/jsonlogs/{model_type.__name__}'
@@ -865,20 +869,24 @@ class BaseExperiment:
         ckpt_dir = f'{self.log_path}/checkpoints/{model_type.__name__}/{model_metadata}' 
         jsonlog_dir = f'{self.log_path}/jsonlogs/{model_type.__name__}/{model_metadata}' 
         tensorboard_dir =  f'{self.log_path}/tensorboardlogs/{model_type.__name__}/{model_metadata}' 
-        if not os.path.exists(ckpt_dir) or not os.path.exists(jsonlog_dir) or not os.path.exists(tensorboard_dir):
-            logger.info(f"There is no such model that is saved.")
-            return None
+        if version > -1:
+            del_ckpt_path = f"{del_ckpt_path}/version_{version}"
+            del_json_path = f"{del_json_path}/version_{version}"
+            del_tensorboard_path = f"{del_tensorboard_path}/version_{version}"
+            ckpt_dir = f"{ckpt_dir}/version_{version}"
+            jsonlog_dir = f"{jsonlog_dir}/version_{version}"
+            tensorboard_dir = f"{tensorboard_dir}/version_{version}"
         
 
         success = {ckpt_dir:0, jsonlog_dir:0, tensorboard_dir:0}
         if os.path.exists(ckpt_dir):
-            shutil.rmtree(del_ckpt_path)
+            shutil.rmtree(del_ckpt_path, ignore_errors=True)
             success.update({ckpt_dir:1})
         if os.path.exists(jsonlog_dir):
-            shutil.rmtree(del_json_path)
+            shutil.rmtree(del_json_path, ignore_errors=True)
             success.update({jsonlog_dir:1})
         if os.path.exists(tensorboard_dir):
-            shutil.rmtree(del_tensorboard_path)
+            shutil.rmtree(del_tensorboard_path, ignore_errors=True)
             success.update({tensorboard_dir:1})
 
         for key, value in success.items():
