@@ -10,7 +10,9 @@ from astrape.utilities.utils_lightning import *
 from astrape.base.experiment_base import BaseExperiment
 from astrape.constants.astrape_constants import *
 from astrape.exceptions.exceptions import *
-
+import logging
+log = logging.getLogger()
+log.setLevel(logging.INFO)
 
 class CrossValidation(BaseExperiment):
     r"""Performs stratified K-fold cross validation.
@@ -59,6 +61,7 @@ class CrossValidation(BaseExperiment):
         mode : Optional[str] = None,
         cv : int = 5,
         search_type : str = 'random',
+        stratify : bool = True,
         n_random_search : Optional[int] = None,
         random_number : int = DEFAULT_RANDOM_SEED_NUMBER
     ):
@@ -91,6 +94,9 @@ class CrossValidation(BaseExperiment):
 
             path (str): Base path for the project. Default is the current directory.
 
+            stratify (bool): Whether to use label-stratification in splitting data into train/val/test set.
+                Default: ``True``
+            
             model_type (`LightningModule`, `BaseEstimator): The type of model (class)
 
             parameters (dict): Dictionary specifying sets of hyperparameters.
@@ -140,7 +146,7 @@ class CrossValidation(BaseExperiment):
         """
         self.rng = np.random.default_rng(random_number)
         self.seed_num = int(10**6 * self.rng.random(1))
-        
+
         super().__init__(
             project_name=project_name,
             X=X,
@@ -151,6 +157,7 @@ class CrossValidation(BaseExperiment):
             test_size=test_size,
             stack_models=stack_models,
             random_number=self.seed_num,
+            stratify=stratify,
             path=path
         )
         self.model_type = model_type
@@ -160,7 +167,11 @@ class CrossValidation(BaseExperiment):
         self.val_metric = val_metric
         self.cv = cv
         self.dims = dims
-
+        self.stratify = stratify
+        if self.n_classes is None and self.stratify:
+            log.info(f"Cannot perform label stratification in regression tasks. Setting stratify to False.")
+            self.stratify = False
+        
         if 'batch_size' not in self.parameters.keys():
             self.parameters.update({'batch_size' : {'values' : [DEFAULT_BATCH_SIZE]}})
         self.jsonlog_dir = None
@@ -273,6 +284,7 @@ class CrossValidation(BaseExperiment):
 
             :obj:`LightningDataModule` or tuple: The datamodule. Returns tuple of numpy ndarrays when using sklearn models.
         """
+        X_train, y_train = (None, None)
         if BaseEstimator in inspect.getmro(self.model_type):
             if (X_val or y_val) and not (X_val and y_val):
                 raise ValueError("X_val and y_val should have the same # of samples.")
@@ -281,14 +293,24 @@ class CrossValidation(BaseExperiment):
             elif X_val and y_val and X_test and y_test:
                 data_module = (X, X_val, y, y_val, X_test, y_test)
             elif X_test and y_test:
-                X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=test_size, random_state=self.random_state, stratify=y)
+                if self.stratify:
+                    X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=test_size, random_state=self.random_state, stratify=y)
+                else:
+                    X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=test_size, random_state=self.random_state)
                 data_module = (X_train, X_val, y_train, y_val, X_test, y_test)
             elif X_val and y_val:
-                X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=test_size, random_state=self.random_state, stratify=y)
-                X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=test_size, random_state=self.random_state, stratify=y)
+                if self.stratify:
+                    X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=test_size, random_state=self.random_state, stratify=y)
+                    X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=test_size, random_state=self.random_state, stratify=y)
+                else:
+                    X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=test_size, random_state=self.random_state)
+                    X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=test_size, random_state=self.random_state)
                 data_module = (X_train, X_val, y_train, y_val, X_test, y_test)
             elif X_val and y_val:
-                X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=test_size, random_state=self.random_state, stratify=y)
+                if self.stratify:
+                    X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=test_size, random_state=self.random_state, stratify=y)
+                else:
+                    X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=test_size, random_state=self.random_state)
                 data_module = (X_train, X_val, y_train, y_val, X_test, y_test)
 
         elif pl.LightningModule in inspect.getmro(self.model_type):
@@ -321,11 +343,11 @@ class CrossValidation(BaseExperiment):
         best_hparams = None
         hparams_dict_list = self.get_hparams_dict_list()
         for hparam_idx, hparams_dict in enumerate(hparams_dict_list):
-            print("Stratified {}-Fold validation with {} set of hyperparameters. Total {} runs will be conducted.".format(self.cv, len(hparams_dict_list), self.cv*len(hparams_dict_list)))
+            log.info(f"Stratified {self.cv}-Fold validation with {len(hparams_dict_list)} set of hyperparameters. Total {self.cv*len(hparams_dict_list)} runs will be conducted.")
             val_score_per_run = []
             for cv_idx, (train_idx, val_idx) in enumerate(cv_indices):
-                print("trying {}/{} set of hyperparameters.".format(hparam_idx+1, len(hparams_dict_list)))
-                print("running {}/{} folds.".format(cv_idx+1, self.cv))
+                log.info(f"trying {hparam_idx+1}/{hparams_dict_list} set of hyperparameters.")
+                log.info(f"running {cv_idx+1}/{self.cv} folds.")
                 X_train, X_val = self.X[train_idx], self.X[val_idx]
                 y_train, y_val = self.y[train_idx], self.y[val_idx]
                 self.model = self.set_model(
